#include "modulator.h"
#define LPFTAPS 255


static short lpfCoeff[LPFTAPS]={
#include "lpf.txt"
};

//		4{C^[|[^[BQC4B^bv32
static short up4Coeff[32]={
#include "interpolate4.txt"
};

//		8{C^[|[^[BQC4B^bv64
static short up8Coeff[64]={
#include "interpolate8.txt"
};

CModulator::CModulator():mcm::fwEzKit2191( 16 )
{
	adder = new stk::CAdder();
	mixer = new stk::CMultiplier();
	osc1 = new stk::CSinCosDDS(2048, 0);		// 1.5KHz
	osc2 = new stk::CSinCosDDS(19413, 21845);	// 455KHz@1536Ksample/s
	
	lpf_i = new stk::CFirFilter( LPFTAPS, lpfCoeff );
	lpf_q = new stk::CFirFilter( LPFTAPS, lpfCoeff );
	
	up8_i = new stk::CUpSampler( 64, up8Coeff, 8 );
	up8_q = new stk::CUpSampler( 64, up8Coeff, 8 );
	
	up4_i = new stk::CUpSampler( 32, up4Coeff, 4 );
	up4_q = new stk::CUpSampler( 32, up4Coeff, 4 );
	
	temp_i = new short[bufSize*32];
	temp_q = new short[bufSize*32];
	temp_ia = new short[bufSize*8];
	temp_qa = new short[bufSize*8];
	
	lo_i = new short[bufSize*32];
	lo_q = new short[bufSize*32];
	in = new short[bufSize];
	
}

CModulator::~CModulator()
{
	delete adder;
	delete mixer;
	delete osc1;
	delete osc2;
	delete lpf_i;
	delete lpf_q;
	delete up4_i;
	delete up4_q;
	delete up8_i;
	delete up8_q;
	
	delete temp_i;
	delete temp_q;
	delete temp_ia;
	delete temp_qa;
	delete lo_i;
	delete lo_q;
	delete in;
}

void CModulator::addFreq( short delta )
{
	osc2->addFreq( delta,0 );
}

//**************************************************************************
//
//  CModulator::handleBuffer̒`
//
//  handleBuffero[֐DMA]I邽тɎMobt@ƑM
//  obt@p[^ɌĂ΂BMobt@ɂ͒ODMAŎM
//  f[^ĂBhandleBuffer߂ƂɑMobt@Ƀf[^
//  ĂƁADMA]őMB
//
//  
//
//**************************************************************************

void CModulator::handleBuffer( struct mcm::sample * bufTx, 
								struct mcm::sample * bufRx )
{
	short step;
	
	step = sizeof(mcm::sample);
				
			// make stereo to monoral
	adder->run( in, &bufRx[0].r, &bufRx[0].l, bufSize, 1, step, step );
	
	osc1->run( lo_i, lo_q, bufSize );			// g
	mixer->run( temp_i, in, lo_i, bufSize );	// MIX
	mixer->run( temp_q, in, lo_q, bufSize );

	lpf_i->run( temp_i, temp_i, bufSize );		// Jbg
	lpf_q->run( temp_q, temp_q, bufSize );

	up8_i->run( temp_ia, temp_i, bufSize );		// 8{I[o[TvO 10%
	up8_q->run( temp_qa, temp_q, bufSize );		// JbgIt=0.06

	up4_i->run( temp_i, temp_ia, bufSize*8 );	// 4{I[o[TvO 35%
	up4_q->run( temp_q, temp_qa, bufSize*8 );	// JbgIt=0.06
	
	osc2->run( lo_i, lo_q, bufSize*32 );			// g			20%
	mixer->run( temp_i, temp_i, lo_i, bufSize*32 );	// MIX
	mixer->run( temp_q, temp_q, lo_q, bufSize*32 );
	
	adder->run( temp_i, temp_i, temp_q, bufSize*32);

}



